varying vec3 vertex,normal;

uniform float level1,level2,step_length;
uniform vec3 box1,box2;

uniform vec3 pos;
uniform vec3 nav;

uniform sampler3D f_text;
uniform sampler3D f_text_i;
uniform sampler3D f_text_n;

uniform float flag;
//uniform float stereo_mode;
uniform float color_scale;
//uniform vec3 level_colors[257];
uniform vec3 level_color1;
uniform vec3 level_color2;

const float z_near=0.3;
const float z_far=60.0;



float Equ(in vec3 arg)
{
	return texture3D(f_text, arg).x;
}



const float delta=1.0/64.0;
vec3 GradEqu(in vec3 arg)
{
		vec3 res;
		res.x = Equ(vec3(arg.x+delta,arg.y,arg.z))-Equ(vec3(arg.x-delta,arg.y,arg.z));
		res.y = Equ(vec3(arg.x,arg.y+delta,arg.z))-Equ(vec3(arg.x,arg.y-delta,arg.z));
		res.z = Equ(vec3(arg.x,arg.y,arg.z+delta))-Equ(vec3(arg.x,arg.y,arg.z-delta));
		return normalize(res);

//		return normalize(texture3D(f_text_n, arg).xyz);
}

#define K_A 0.1
#define K_D 0.8
#define K_S 0.8
#define P 64.0

vec3 Phong ( vec3 point, vec3 normal, vec3 color)
{
   	vec3 light = normalize( pos - point );
	float diffuse = dot ( light, normal );
	vec3 reflect = reflect ( -light, normal );
	float specular = pow ( max (0.0, dot ( reflect, light ) ), P );
	return K_A * vec3(1.0) + diffuse * ( K_D * color + K_S * specular * vec3(1.0) );
}

vec3 GetLevelColor(float l)
{
	/*
	float fl = 256.0*l;
	int id = int(fl);
	return mix(level_colors[id],level_colors[id+1],floor(fl));
	*/
	//l*=color_scale;
	return mix(level_color1,level_color2,l);//vec3(l,0,1.0-l);//vec3(sin(l),cos(l*2.0),sin(l*3.0))*0.5+vec3(0.5);
}
/*
float IntersectBox ( vec3 b1, vec3 b2, vec3 pos, vec3 nav,out vec3 stp)
{
	vec3 OMAX = ( b1 - pos ) / nav;
   
	vec3 OMIN = ( b2 - pos ) / nav;
	
	vec3 MAX = max ( OMAX, OMIN );
	
	float res = min ( MAX.x, min ( MAX.y, MAX.z ) );
	stp = vec3( (res==MAX.x)?1.0:0.0 , (res==MAX.y)?1.0:0.0 , (res==MAX.z)?1.0:0.0 );
	return res;
}

const float ICellSize = 16.0/256.0;
*/
void main()
{
	float alpha = 0.0;
	vec3 color = vec3(0.0);
	vec3 norm=normal, ray = normalize(vertex-pos);
	vec3 step  = ray*step_length;
	vec3 stepi  = step*6.0;
	
	vec3 ps=vertex;
	vec2 e2;

	do
	{
		ps+=stepi;
		e2 = texture3D(f_text_i, ps).xy;
	}while((ps == clamp(ps,box1,box2)) && (level1>e2.y || level2<e2.x));
	
	ps-=stepi;
	
	float e = Equ(ps),e0;
	
	ps+=step;
	//if((level2>1.0) && (!(level1>e2.y || level2<e2.x))){alpha=1.0;color=vec3(1.0);}else
	{
	
	if(e == clamp(e,level1,level2))
	{
		color = GetLevelColor(e);
		alpha = 1.0;
		ps -= pos;	gl_FragDepth = z_far/(z_far-z_near) - (z_far*z_near/(z_far-z_near))/(dot(ps,nav));
	}else
	{
		gl_FragDepth=1.0;
		
		if(e < level1)
		{
			while(ps == clamp(ps,box1,box2))
			{
				e0=e;
				e = Equ(ps);
				
				if(e >= level1)
				{
					ps += step*((level1-e)/(e-e0));
					norm = -GradEqu(ps);
					color = level_color1;
					alpha = 1.0;
					ps -= pos;	gl_FragDepth = z_far/(z_far-z_near) - (z_far*z_near/(z_far-z_near))/(dot(ps,nav));
					break;
				}
				ps+=step;
			}
		}else
		if(e > level2)
		{
			while(ps == clamp(ps,box1,box2))
			{
				e0=e;
				e = Equ(ps);
				
				if(e <= level2)
				{
					ps += step*((level2-e)/(e-e0));
					norm = GradEqu(ps);
					color = level_color2;
					alpha = 1.0;
					ps -= pos;	gl_FragDepth = z_far/(z_far-z_near) - (z_far*z_near/(z_far-z_near))/(dot(ps,nav));
					break;
				}
					
				ps+=step;
			}
				
		}

			
	}
	}
	
	
	gl_FragColor = vec4(Phong(ps,norm,color),alpha);
	
}
